/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * AbstractExternalListener.java - Initial Developper : ofabre
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.component.framework.listener;

import javax.jbi.messaging.MessagingException;

import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.jbidescriptor.generated.Consumes;

/**
 * The abstract methods for an external listener.
 * 
 * @author alouis, ofabre - EBM Websourcing
 * 
 */
public abstract class AbstractExternalListener extends AbstractListener {

    /**
     * Starts the listener. This method is called when the SU that causes this
     * listener instantiation is started.
     * 
     * @throws PEtALSCDKException
     *             if an error occurs during external listener starting process
     */
    public abstract void start() throws PEtALSCDKException;

    /**
     * Stops the listener. This method is called when the SU that causes this
     * listener instantiation is stopped.
     * 
     * @throws PEtALSCDKException
     *             if an error occurs during external listener stopping process
     */
    public abstract void stop() throws PEtALSCDKException;

    /**
     * Create a message exchange with all the information set in the related
     * {@link Consumes} Su configuration.<br>
     * - service information (itf, srv, ep) <br>
     * - MEP <br>
     * - operation <br>
     * - Properties
     * 
     * @return The message exchange
     * @throws MessagingException
     *             The message exchange can't be created or no pattern specified
     *             in extensions
     * @throws PEtALSCDKException
     *             The current Transaction has failed to return its status
     * @throws PEtALSCDKException
     */
    public Exchange createConsumeExchange() throws MessagingException, PEtALSCDKException {
        return this.createConsumeExchange(getConsumes());
    }
}
