/**
 * PETALS - PETALS Services Platform. Copyright (c) 2005 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * ---------------------------------------------------------------------------
 * ---------------------------------------------------------------------------
 */
package org.ow2.petals.component.framework.notification.filter.messagecontent;

import java.util.List;
import java.util.concurrent.CopyOnWriteArrayList;

import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.ow2.petals.component.framework.notification.filter.AbstractFilter;
import org.w3c.dom.Document;

import com.ebmwebsourcing.wsstar.notification.definition.basenotification.api.QueryExpressionType;
import com.ebmwebsourcing.wsstar.notification.definition.basenotification.api.Subscribe;
import com.ebmwebsourcing.wsstar.notification.definition.topics.WstopConstants;
import com.ebmwebsourcing.wsstar.notification.definition.utils.WSNotificationException;
import com.ebmwebsourcing.wsstar.notification.extension.utils.WSNotificationExtensionException;
import com.ebmwebsourcing.wsstar.notification.extension.utils.WsnSpecificTypeHelper;

/**
 * A filter on the content of the exchange
 * 
 * @author Frederic Gardes
 * 
 */
public class XPathFilter extends AbstractFilter {

    private XPath xPath = null;

    private List<XPathExpression> xpathExpressionList = null;

    public XPathFilter(final Subscribe subscribe) throws XPathExpressionException,
    WSNotificationExtensionException, WSNotificationException {
        super(subscribe);
        String xPathString = null;
        XPathExpression xpathExpression = null;

        // TODO not test all the MessageContent
        if (this.filterType != null) {
            final List<QueryExpressionType> queryExpressionList = this.filterType
            .getMessageContentList();
            for (final QueryExpressionType queryExpression : queryExpressionList) {
                if (queryExpression.getDialect().equals(
                        WstopConstants.XPATH_TOPIC_EXPRESSION_DIALECT_URI)) {
                    // create the XPath if needed
                    if (this.xPath == null) {
                        this.xPath = XPathFactory.newInstance().newXPath();
                    }
                    // create the XPath expression list if needed
                    if (this.xpathExpressionList == null) {
                        this.xpathExpressionList = new CopyOnWriteArrayList<XPathExpression>();
                    }
                    xPathString = WsnSpecificTypeHelper.getXpathExpression(queryExpression);
                    if (xPathString != null) {
                        xpathExpression = this.xPath.compile(xPathString);
                        if (xpathExpression != null) {
                            this.xpathExpressionList.add(xpathExpression);
                        }
                    }
                }
            }
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.notification.filter.Filter#isNotifiable
     * (com.ebmwebsourcing.wsstar.notification.definition.basenotification.api.
     * FilterType, java.lang.Object)
     */
    public boolean isNotifiable(final Object content) {
        String result = null;
        Document document = null;

        try {
            if (this.xPath != null && this.xpathExpressionList != null) {
                for (final XPathExpression xpathExpression : this.xpathExpressionList) {
                    if (content == null || !(content instanceof Document)) {
                        return false;
                    }
                    document = (Document) content;
                    result = xpathExpression.evaluate(document);
                    this.xPath.reset();
                    if (result == null || result.length() == 0 || result.equals("false")) {
                        return false;
                    }
                }
            }
        } catch (final XPathExpressionException e) {
            return false;
        }
        return true;
    }

}
