/**
 * PETALS - PETALS Services Platform. Copyright (c) 2005 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * ---------------------------------------------------------------------------
 * ---------------------------------------------------------------------------
 */
package org.ow2.petals.component.framework.notification.notify;

import java.util.logging.Logger;

import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactoryConfigurationError;
import javax.xml.xpath.XPathExpressionException;

import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.api.notification.NotificationBrokerController;
import org.ow2.petals.component.framework.util.UtilFactory;
import org.w3c.dom.Document;

import com.ebmwebsourcing.wsstar.addressing.definition.api.EndpointReferenceType;
import com.ebmwebsourcing.wsstar.notification.definition.basenotification.api.Subscribe;
import com.ebmwebsourcing.wsstar.notification.definition.utils.WSNotificationException;
import com.ebmwebsourcing.wsstar.notification.extension.utils.WSNotificationExtensionException;

/**
 * The notification of an out message on a JBI Producer
 * 
 * @author Frederic Gardes
 * 
 */
public class OutProducerComponentNotify extends AbstractProducerComponentNotify {

    /**
     * @param subscriptionReference
     * @param subscribe
     * @param notificationBrokerController
     * @param logger
     * @throws WSNotificationException
     * @throws WSNotificationExtensionException
     * @throws XPathExpressionException
     * @throws TransformerFactoryConfigurationError
     * @throws TransformerException
     * @throws ParserConfigurationException
     */
    public OutProducerComponentNotify(final EndpointReferenceType subscriptionReference,
            final Subscribe subscribe,
            final NotificationBrokerController notificationBrokerController, final Logger logger)
    throws WSNotificationException, WSNotificationExtensionException,
    XPathExpressionException, TransformerFactoryConfigurationError, TransformerException,
    ParserConfigurationException {
        super(subscriptionReference, subscribe, notificationBrokerController, logger);
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.notification.generation.
     * AbstractProducerComponentNotify#getTopic()
     */
    @Override
    public String getTopic() {
        return "petals:component/cdk/producer/out[@wstop:topic='true']";
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.notification.notify.
     * AbstractProducerComponentNotify
     * #process(org.ow2.petals.component.framework.api.message.Exchange)
     */
    @Override
    public synchronized Exchange process(final Exchange exchange) throws MessagingException {
        Document document = null;
        NormalizedMessage normalizedMessage = null;

        try {
            if (exchange.isActiveStatus() && exchange.getFault() == null) {
                normalizedMessage = exchange.getOutMessage();
                if (normalizedMessage != null) {
                    document = UtilFactory.getSourceUtil().createDocument(
                            exchange.getOutMessage().getContent());
                    if (document != null) {
                        this.contextSubscriptionPolicy.setStatus(0);
                        this.process(exchange, document);
                    }
                }
            }
        } catch (final PEtALSCDKException e) {
            this.logger.info("Unable to send a notification");
            this.logger.fine("Detail on the notification error: " + e.getMessage());
        } finally {
            if (document != null) {
                try {
                    exchange.getOutMessage().setContent(
                            UtilFactory.getSourceUtil().createStreamSource(document));
                } catch (final PEtALSCDKException e) {
                    // nothing to do
                }
            }
        }
        return exchange;
    }
}
