/**
 * PETALS - PETALS Services Platform. Copyright (c) 2005 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * ---------------------------------------------------------------------------
 * ---------------------------------------------------------------------------
 */
package org.ow2.petals.component.framework.notification.notify;

import java.util.logging.Logger;

import javax.jbi.messaging.MessagingException;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactoryConfigurationError;
import javax.xml.xpath.XPathExpressionException;

import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.api.notification.NotificationBrokerController;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import com.ebmwebsourcing.wsstar.addressing.definition.api.EndpointReferenceType;
import com.ebmwebsourcing.wsstar.notification.definition.basenotification.api.Subscribe;
import com.ebmwebsourcing.wsstar.notification.definition.utils.WSNotificationException;
import com.ebmwebsourcing.wsstar.notification.extension.utils.WSNotificationExtensionException;

/**
 * The notification of an exchange with a Done or Error status on a JBI Producer
 * 
 * @author Frederic Gardes
 * 
 */
public class StatusProducerComponentNotify extends AbstractProducerComponentNotify {

    /**
     * @param subscriptionReference
     * @param subscribe
     * @param notificationBrokerController
     * @param logger
     * @throws WSNotificationException
     * @throws WSNotificationExtensionException
     * @throws XPathExpressionException
     * @throws TransformerFactoryConfigurationError
     * @throws TransformerException
     * @throws ParserConfigurationException
     */
    public StatusProducerComponentNotify(final EndpointReferenceType subscriptionReference,
            final Subscribe subscribe,
            final NotificationBrokerController notificationBrokerController, final Logger logger)
    throws WSNotificationException, WSNotificationExtensionException,
    XPathExpressionException, TransformerFactoryConfigurationError, TransformerException,
    ParserConfigurationException {
        super(subscriptionReference, subscribe, notificationBrokerController, logger);
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.notification.generation.
     * AbstractProducerComponentNotify#getTopic()
     */
    @Override
    public String getTopic() {
        return "petals:component/cdk/producer/status[@wstop:topic='true']";
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.notification.generation.
     * AbstractProducerComponentNotify
     * #process(org.ow2.petals.component.framework.api.message.Exchange)
     */
    @Override
    public synchronized Exchange process(final Exchange exchange) throws MessagingException {
        Document document = null;
        Element root = null;
        try {
            if (exchange.isErrorStatus() || exchange.isDoneStatus()) {
                // TODO create a DocumentUtil, static singleton to do this only
                // one time
                final DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory
                .newInstance();
                final DocumentBuilder documentBuilder = documentBuilderFactory.newDocumentBuilder();
                document = documentBuilder.newDocument();
                if (document != null) {
                    if (exchange.isErrorStatus()) {
                        root = document.createElement("exception");
                        if (exchange.getError() == null || exchange.getError().getMessage() == null) {
                            root.setTextContent("An unknown exception append");
                        } else {
                            root.setTextContent(exchange.getError().getMessage());
                        }
                        this.contextSubscriptionPolicy.setStatus(1);
                    } else {
                        root = document.createElement("status");
                        this.contextSubscriptionPolicy.setStatus(0);
                    }
                    document.appendChild(root);
                    this.process(exchange, document);
                }
                documentBuilder.reset();
            }
        } catch (final ParserConfigurationException e) {
            this.logger.info("Unable to send a notification");
            this.logger.fine("Detail on the notification error: " + e.getMessage());
        }
        return exchange;
    }

}
