/**
 * PETALS - PETALS Services Platform. Copyright (c) 2005 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.component.framework.util;

import java.io.IOException;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import javax.jbi.servicedesc.ServiceEndpoint;
import javax.xml.namespace.QName;

import org.ow2.easywsdl.extensions.wsdl4complexwsdl.WSDL4ComplexWsdlFactory;
import org.ow2.easywsdl.extensions.wsdl4complexwsdl.api.Description;
import org.ow2.easywsdl.extensions.wsdl4complexwsdl.api.WSDL4ComplexWsdlReader;
import org.ow2.easywsdl.extensions.wsdl4complexwsdl.api.WSDL4ComplexWsdlWriter;
import org.ow2.easywsdl.wsdl.WSDLFactory;
import org.ow2.easywsdl.wsdl.api.Endpoint;
import org.ow2.easywsdl.wsdl.api.InterfaceType;
import org.ow2.easywsdl.wsdl.api.Service;
import org.ow2.easywsdl.wsdl.api.WSDLException;
import org.ow2.easywsdl.wsdl.api.abstractItf.AbsItfDescription.WSDLVersionConstants;
import org.ow2.petals.component.framework.api.Constants;
import org.ow2.petals.component.framework.api.util.EndpointUtil;
import org.ow2.petals.component.framework.api.util.WSDLUtil;
import org.w3c.dom.Document;

/**
 * Util class to manipulate WSDL Document or Definition.
 * 
 * @author wjoseph - EBMWebsourcing
 * @since 1.0
 */
public final class WSDLUtilImpl implements WSDLUtil {

    private WSDL4ComplexWsdlFactory wsdlFactory = null;

    private WSDL4ComplexWsdlWriter wsdlWriter = null;

    private WSDL4ComplexWsdlReader wsdlReader = null;

    /**
     * Creates a new instance of {@link WSDLUtilImpl}
     * 
     * @throws WSDLException
     * 
     */
    public WSDLUtilImpl() throws WSDLException {
        this.wsdlFactory = WSDL4ComplexWsdlFactory.newInstance();
        this.wsdlReader = this.wsdlFactory.newWSDLReader();
        this.wsdlWriter = this.wsdlFactory.newWSDLWriter();
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.util.IWSDLUtil#createWSDL20Endpoint
     * (javax.jbi.servicedesc.ServiceEndpoint)
     */
    public final Endpoint createWSDL20Endpoint(final ServiceEndpoint serviceEndpoint)
            throws WSDLException {
        final Description wsdlDescription = this.createLightWSDL20Description(serviceEndpoint
                .getInterfaces()[0], serviceEndpoint.getServiceName(), serviceEndpoint
                .getEndpointName());
        return wsdlDescription.getServices().get(0).getEndpoints().get(0);
    }

    /**
     * Create a light WSDL2.0 description from the given parameters. All
     * parameters are required, otherwise a NullPointerException will be thrown.
     * Namespaces of the WSDL is managed, and the namespace of the service and
     * the interface must be the same, as we do not manage imported namespace in
     * the light WSDL.<br >
     * The service contains the unique endpoint. The service implements an
     * unique interface.
     * 
     * @param interfaceName
     *            must be non null
     * @param serviceName
     *            must be non null
     * @param endpointName
     *            must be non null
     * @return a WSDL2.0 description
     * @throws WSDLException
     */
    public final Description createLightWSDL20Description(final QName interfaceQName,
            final QName serviceQName, final String endpointName) throws WSDLException {

        if (!interfaceQName.getNamespaceURI().equals(serviceQName.getNamespaceURI())) {
            throw new WSDLException("The interface namespace '" + interfaceQName.getNamespaceURI()
                    + "' must be the same than the service namespace '"
                    + serviceQName.getNamespaceURI() + "' to build a light WSDL");
        }

        final Description wsdlDescription = this.wsdlFactory
                .addComplexWsdlElmt2Description(WSDLFactory.newInstance().newDescription(
                        WSDLVersionConstants.WSDL20));
        wsdlDescription.setTargetNamespace(interfaceQName.getNamespaceURI());
        wsdlDescription.setQName(new QName("autoGenerated"));
        
        final InterfaceType interfazz = wsdlDescription.createInterface();
        interfazz.setQName(interfaceQName);
        wsdlDescription.addInterface(interfazz);
        
        final Service service = wsdlDescription.createService();
        service.setQName(serviceQName);
        service.setInterface(interfazz);
        
        final Endpoint endpoint = service.createEndpoint();
        endpoint.setName(endpointName);
        service.addEndpoint(endpoint);
        wsdlDescription.addService(service);
        
        return wsdlDescription;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.util.WSDLUtil#getEndpointList(
     * org.ow2.easywsdl.extensions.wsdl4complexwsdl.api.Description,
     * javax.xml.namespace.QName, java.lang.String)
     */
    public List<Endpoint> getEndpointList(final Description wsdlDescription,
            final QName serviceQNameFilter, final String endpointNameFilter) {
        final List<Endpoint> endpointList = new ArrayList<Endpoint>();
        Endpoint endpoint = null;
        if (wsdlDescription != null) {
            if (serviceQNameFilter == null) {
                final List<Service> serviceList = wsdlDescription.getServices();
                for (final Service service : serviceList) {
                    if (service != null) {
                        if (endpointNameFilter == null) {
                            endpointList.addAll(service.getEndpoints());
                        } else {
                            endpoint = service.getEndpoint(endpointNameFilter);
                            if (endpoint != null) {
                                endpointList.add(service.getEndpoint(endpointNameFilter));
                            }
                        }
                    }
                }
            } else {
                final Service service = wsdlDescription.getService(serviceQNameFilter);
                if (service != null) {
                    if (endpointNameFilter == null) {
                        endpointList.addAll(service.getEndpoints());
                    } else {
                        endpoint = service.getEndpoint(endpointNameFilter);
                        if (endpoint != null) {
                            endpointList.add(service.getEndpoint(endpointNameFilter));
                        }
                    }
                }
            }
        }
        return endpointList;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.util.IWSDLUtil#getEndpointList(org
     * .ow2.easywsdl.extensions.wsdl4complexwsdl.api.Description)
     */
    public List<Endpoint> getEndpointList(final Description wsdlDescription) {
        return this.getEndpointList(wsdlDescription, null, null);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.util.IWSDLUtil#createWsdlDescription
     * (java.net.URL, boolean, boolean)
     */
    public Description createWsdlDescription(final URL wsdlLocation,
            final boolean autogenerateEndpoint) throws WSDLException {
        String autogeneratedEndpointName = null;
        if (wsdlLocation == null) {
            return null;
        }
        try {
            final Description description = this.wsdlReader.read(wsdlLocation);
            description.addImportedDocumentsInWsdl();
            if (autogenerateEndpoint) {
                final List<Endpoint> endpointList = this.getEndpointList(description, null,
                        Constants.Component.AUTOGENERATED_ENDPOINT_NAME);
                for (final Endpoint endpoint : endpointList) {
                    autogeneratedEndpointName = EndpointUtil.generateEndpointName();
                    endpoint.setName(autogeneratedEndpointName);
                }
            }

            return description;
        } catch (final URISyntaxException e) {
            throw new WSDLException("Failed to parse the WSDL", e);
        } catch (final IOException e) {
            throw new WSDLException("Failed to read the WSDL", e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.util.IWSDLUtil#
     * convertDescriptionToDocument
     * (org.ow2.easywsdl.extensions.wsdl4complexwsdl.api.Description)
     */
    public Document convertDescriptionToDocument(final Description wsdlDescription)
            throws WSDLException {
        return this.wsdlWriter.getDocument(wsdlDescription);
    }

    public Description convertDocumentToDescription(final Document document) throws WSDLException {
        try {
            return this.wsdlReader.read(document);
        } catch (final URISyntaxException e) {
            throw new WSDLException("Failed to convert the WSDL", e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.util.IWSDLUtil#convertDescriptionToString
     * (org.ow2.easywsdl.extensions.wsdl4complexwsdl.api.Description)
     */
    public String convertDescriptionToString(final Description wsdlDescription)
            throws WSDLException {
        return this.wsdlWriter.writeWSDL4ComplexWsdl(wsdlDescription);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.util.IWSDLUtil#isDescriptionContaining
     * (org.ow2.easywsdl.extensions.wsdl4complexwsdl.api.Description,
     * javax.jbi.servicedesc.ServiceEndpoint)
     */
    public boolean isDescriptionContaining(final Description wsdlDescription,
            final ServiceEndpoint endpoint) {
        if (endpoint == null || endpoint.getServiceName() == null
                || endpoint.getEndpointName() == null) {
            return false;
        }
        return !this.getEndpointList(wsdlDescription, endpoint.getServiceName(),
                endpoint.getEndpointName()).isEmpty();
    }
}
